/*
 * Copyright (c) 2020, Max Klein
 *
 * This file is part of the tree-finder library, distributed under the terms of
 * the BSD 3 Clause license. The full license can be found in the LICENSE file.
 */
import * as rt from "regular-table";
import { Format, RegularTable, Tree } from "../util";
// await customElements.whenDefined('regular-table');
if (document.createElement("regular-table").constructor === HTMLElement) {
    window.customElements.define("regular-table", rt.RegularTableElement);
}
const RegularTableElement = customElements.get('regular-table');
export class TreeFinderGridElement extends RegularTableElement {
    constructor() {
        super(...arguments);
        this._initializedListeners = false;
    }
    async init(model, options = {}) {
        this.model = model;
        this.options = options;
        // TODO: fix the godawful typing of .setDataListener on the regular-table side
        this.setDataListener(this.dataListener.bind(this), { virtual_mode: this._options.virtual_mode });
        this.model.drawSub.subscribe(async (x) => {
            if (x) {
                this._resetAutoSize();
            }
            await this.draw();
            if (this.model.options.needsWidths) {
                // correct handling of autowidth seems to frequently require the second draw
                await this.draw();
            }
        });
        // run listener initializations only once
        if (!this._initializedListeners) {
            const [thead, tbody] = [this.table_model.header.table, this.table_model.body.table];
            this.addStyleListener(() => this.columnHeaderStyleListener());
            this.addStyleListener(() => this.rowStyleListener());
            this.addEventListener("mouseover", event => this.onMouseover(event));
            thead.addEventListener("mouseup", event => this.onSortClick(event));
            tbody.addEventListener("dblclick", event => this.onRowDoubleClick(event));
            tbody.addEventListener("mouseup", event => this.onRowClick(event));
            tbody.addEventListener("mouseup", event => this.onTreeClick(event));
            // this.addEventListener("scroll", () => (this as any)._resetAutoSize());
            // click debug listener
            // this.addEventListener("mousedown", event => RegularTable.clickLoggingListener(event, this));
            if (this.options.doWindowResize) {
                // resize whenever window size changes, if requested
                window.addEventListener('resize', async () => {
                    await this.draw();
                });
            }
            this._initializedListeners = true;
        }
        this.model.requestDraw();
    }
    async dataListener(start_col, start_row, end_col, end_row) {
        var _a, _b, _c;
        const data = [];
        for (const column of this.model.columns.slice(start_col, end_col)) {
            const formatter = (_c = (_b = (_a = this.options) === null || _a === void 0 ? void 0 : _a.columnFormatters) === null || _b === void 0 ? void 0 : _b[column]) !== null && _c !== void 0 ? _c : (x => x);
            data.push(this.model.contents.slice(start_row, end_row).map(content => {
                const val = formatter(content.row[column]);
                return (val instanceof Date ? Format.DATE_FORMATTER.format(val) : val);
            }));
        }
        return {
            // num_columns/rows: number -> count of cols/rows
            num_columns: this.model.columns.length,
            num_rows: this.model.contents.length,
            // column/row_headers: string[] -> arrays of path parts that get displayed as the first value in each col/row. Length > 1 implies a tree structure
            column_headers: this.model.columns.slice(start_col, end_col).map((x, i) => this.options.showFilter ? [this.filters.getChild(i + 1), ...Tree.colHeaderSpans(x)] : Tree.colHeaderSpans(x)),
            row_headers: this.model.contents.slice(start_row, end_row).map(x => {
                return Tree.rowHeaderSpan({
                    isDir: x.isDir,
                    isOpen: x.isExpand,
                    path: x.getPathAtDepth(this.model.pathDepth),
                    pathRender: this._pathRender,
                });
            }),
            // data: object[][] -> array of arrays, each subarray containing all of the visible values for one column
            data,
        };
    }
    columnHeaderStyleListener() {
        var _a, _b;
        this.cornerHeaderStyleListener();
        for (const th of this.querySelectorAll("thead tr:first-child th")) {
            th.classList.toggle("tf-header-align-left", true);
            const filter = th.querySelector("tree-finder-filter");
            if (filter) {
                filter.input.classList.toggle("tf-header-input", true);
                if (this.model.filterCol === filter.col) {
                    filter.input.focus();
                    delete this.model.filterCol;
                }
            }
        }
        for (const th of this.querySelectorAll("thead tr:last-child th")) {
            const meta = this.getMeta(th);
            const col = RegularTable.colNameFromMeta(meta);
            if (col) {
                const sortOrder = (_a = this.model.sortStates.byColumn[col === "0" ? "path" : col]) === null || _a === void 0 ? void 0 : _a.order;
                (_b = th.querySelector(".tf-header-sort")) === null || _b === void 0 ? void 0 : _b.classList.toggle(`tf-header-sort-${sortOrder}`, !!sortOrder);
            }
            th.classList.toggle("tf-header-corner", typeof meta.x === "undefined");
            th.classList.toggle("tf-header-align-left", true);
        }
        if (this.model.options.needsWidths) {
            this.columnWidthStyleListener();
        }
    }
    columnWidthStyleListener() {
        // complete and accurate autowidth for the tree-finder-filter input elems
        let pxs = [...this._column_sizes.indices];
        if (this._pathRender === "regular" && pxs.length > 0) {
            pxs = [pxs[0] + pxs[1], ...pxs.slice(2)];
        }
        this.model.columnWidthsSub.next(pxs.map(px => `calc(${px}px - 12px)`));
    }
    cornerHeaderStyleListener() {
        const initSpans = this.querySelectorAll(`thead tr > th:first-child > span:first-child:not([class])`);
        const newSpans = this.options.showFilter ? [this.filters.getChild(0), ...Tree.colHeaderSpans("path")] : Tree.colHeaderSpans("path");
        for (let i = 0; i < initSpans.length; i++) {
            initSpans[i].replaceWith(newSpans[i]);
        }
    }
    rowStyleListener() {
        const spans = this.querySelectorAll("tbody th .rt-group-name");
        for (const span of spans) {
            // style the browser's filetype icons
            const { value, y } = RegularTable.metadataFromElement(span, this);
            if (value) {
                const content = this.model.contents[y];
                span.classList.add("tf-grid-filetype-icon", `tf-grid-${content.row.kind}-icon`);
                this.rowSelectStyleListener();
            }
        }
    }
    rowSelectStyleListener() {
        const colCount = this.model.columns.length + 1;
        for (let tr of this.table_model.body.rows) {
            const { y } = RegularTable.metadataFromElement(tr.children[0], this);
            if (tr && tr.tagName === "TR") {
                tr.classList.toggle("tf-mod-select", this.model.selection.has(this.model.contents[y]));
                if (this._pathRender === "regular") {
                    for (let i = 0; i < tr.children.length - colCount; i++) {
                        tr.children[0].classList.toggle("tf-mod-select-not", true);
                    }
                }
            }
        }
    }
    async onMouseover(event) {
        var _a;
        if (this._pathRender === "regular") {
            const element = event.target;
            const metadata = RegularTable.metadataFromElement(element, this);
            const rowHeader = (_a = metadata === null || metadata === void 0 ? void 0 : metadata.row_header) === null || _a === void 0 ? void 0 : _a[0];
            for (let tr of this.table_model.body.rows) {
                if (tr.childElementCount > (this.model.columns.length + 1)) {
                    const th = tr.children[0];
                    th.classList.toggle("tf-mod-hover", th.textContent === rowHeader);
                }
            }
        }
    }
    async onRowClick(event) {
        if (event.button !== 0) {
            return;
        }
        const element = event.target;
        if (element.classList.contains("rt-row-header-icon")) {
            // if open/close icon is clicked, don't select
            return;
        }
        const metadata = RegularTable.metadataFromElement(element, this);
        if (!metadata || !RegularTable.rowClicked(metadata)) {
            return;
        }
        const content = this.model.contents[metadata.y];
        if (event.shiftKey) {
            this.model.selection.selectRange(content, this.model.contents);
        }
        else {
            this.model.selection.select(content, event.ctrlKey || event.metaKey);
        }
        this.rowSelectStyleListener();
        // can't call draw directly or indirectly, breaks any subsequent doubleClick event
        // setTimeout(() => this.model.requestDraw(), 200);
    }
    async onSortClick(event) {
        if (event.button !== 0) {
            return;
        }
        const element = event.target;
        if (element.classList.contains("rt-column-resize")) {
            // don't sort when the column resize handle nodes are clicked
            return;
        }
        const meta = RegularTable.metadataFromElement(element, this);
        if (!meta || !RegularTable.columnHeaderClicked(meta) || meta.column_header_y < this.table_model.header.rows.length - 1) {
            return;
        }
        await this.model.sort({
            col: RegularTable.colNameFromMeta(meta),
            multisort: event.shiftKey,
        });
    }
    async onTreeClick(event) {
        if (event.button !== 0) {
            return;
        }
        let element = event.target;
        if (!element.classList.contains("rt-row-header-icon")) {
            // only open/close node when open/close icon is clicked
            return;
        }
        // assert that metadata exists, given element.classList check above
        const metadata = RegularTable.metadataFromElement(element, this);
        if (this.model.contents[metadata.y].isExpand) {
            await this.model.collapse(metadata.y);
        }
        else {
            await this.model.expand(metadata.y);
        }
        // (this as any)._resetAutoSize();
        // this.draw();
    }
    async onRowDoubleClick(event) {
        if (event.button !== 0) {
            return;
        }
        const element = event.target;
        const metadata = RegularTable.metadataFromElement(element, this);
        if (!metadata || !RegularTable.rowClicked(metadata)) {
            return;
        }
        // event.preventDefault();
        // event.stopPropagation();
        // event.stopImmediatePropagation();
        // event.returnValue = false;
        const newRootContent = this.model.contents[metadata.y];
        if (newRootContent.isDir) {
            await this.model.open(newRootContent.row);
            // .init() calls .draw()
            this.init(this.model, this.options);
        }
    }
    get options() {
        return { ...this._options };
    }
    set options({ columnFormatters = undefined, doWindowResize = false, pathRender = "tree", pathRenderOnFilter = "regular", showFilter = false, virtual_mode = "vertical", }) {
        this._options = {
            columnFormatters,
            doWindowResize,
            pathRender,
            pathRenderOnFilter,
            showFilter,
            virtual_mode,
        };
        this.showFilter = this._options.showFilter;
    }
    set showFilter(flag) {
        if (flag) {
            this.filters = document.createElement("tree-finder-filters");
            this.filters.init(this.model);
        }
        else {
            delete this.filters;
        }
        this._options.showFilter = flag;
    }
    get _pathRender() {
        var _a, _b;
        if (this.model.filterPatterns.any) {
            return ((_a = this.options) === null || _a === void 0 ? void 0 : _a.pathRenderOnFilter) || "tree";
        }
        else {
            return ((_b = this.options) === null || _b === void 0 ? void 0 : _b.pathRender) || "tree";
        }
    }
}
customElements.define("tree-finder-grid", TreeFinderGridElement);
//# sourceMappingURL=grid.js.map