export var Format;
(function (Format) {
    Format.DATE_FORMATTER = new Intl.DateTimeFormat("en-us");
    /**
     * Format bytes as human-readable text.
     * ref: https://stackoverflow.com/a/14919494
     *
     * @param bytes Number of bytes.
     * @param si True to use metric (SI) units, aka powers of 1000. False to use
     *           binary (IEC), aka powers of 1024.
     * @param dp Number of decimal places to display.
     *
     * @return Formatted string.
     */
    function bytesToHumanReadable(bytes, si = false, dp = 1) {
        if (!bytes) {
            return "--";
        }
        const thresh = si ? 1000 : 1024;
        if (Math.abs(bytes) < thresh) {
            return bytes + " B";
        }
        const units = si
            ? ["kB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB"]
            : ["KiB", "MiB", "GiB", "TiB", "PiB", "EiB", "ZiB", "YiB"];
        let u = -1;
        const r = 10 ** dp;
        do {
            bytes /= thresh;
            ++u;
        } while (Math.round(Math.abs(bytes) * r) / r >= thresh && u < units.length - 1);
        return bytes.toFixed(dp) + ' ' + units[u];
    }
    Format.bytesToHumanReadable = bytesToHumanReadable;
    function dateToStr(date, str) {
        return Format.DATE_FORMATTER.format(date).includes(str);
    }
    Format.dateToStr = dateToStr;
    function timeSince(date) {
        date = !(date instanceof Date) ? new Date(date) : date;
        const seconds = Math.floor((Date.now() - date.valueOf()) / 1000);
        let interval = seconds / 31536000;
        if (interval > 1) {
            return Math.floor(interval) + " years";
        }
        interval = seconds / 2592000;
        if (interval > 1) {
            return Math.floor(interval) + " months";
        }
        interval = seconds / 86400;
        if (interval > 1) {
            return Math.floor(interval) + " days";
        }
        interval = seconds / 3600;
        if (interval > 1) {
            return Math.floor(interval) + " hours";
        }
        interval = seconds / 60;
        if (interval > 1) {
            return Math.floor(interval) + " minutes";
        }
        return Math.floor(seconds) + " seconds";
    }
    Format.timeSince = timeSince;
})(Format || (Format = {}));
export var RegularTable;
(function (RegularTable) {
    const TOP_LEVEL_TAGNAME = "TREE-FINDER-GRID";
    /*
     * click-related functions
     */
    function cellClicked(metadata) {
        return typeof (metadata === null || metadata === void 0 ? void 0 : metadata.y) !== "undefined" && !(metadata === null || metadata === void 0 ? void 0 : metadata.column_header_y);
    }
    RegularTable.cellClicked = cellClicked;
    function columnHeaderClicked(metadata) {
        var _a;
        return typeof ((_a = metadata) === null || _a === void 0 ? void 0 : _a.column_header_y) !== "undefined" && !!(metadata === null || metadata === void 0 ? void 0 : metadata.column_header);
    }
    RegularTable.columnHeaderClicked = columnHeaderClicked;
    function rowClicked(metadata) {
        return cellClicked(metadata) || rowHeaderClicked(metadata);
    }
    RegularTable.rowClicked = rowClicked;
    function rowHeaderClicked(metadata) {
        var _a;
        return typeof ((_a = metadata) === null || _a === void 0 ? void 0 : _a.row_header_x) !== "undefined" && !!(metadata === null || metadata === void 0 ? void 0 : metadata.row_header);
    }
    RegularTable.rowHeaderClicked = rowHeaderClicked;
    /*
     * debug utils
     */
    function clickLoggingListener(event, rt) {
        const metadata = metadataFromElement(event.target, rt, true);
        if (!metadata) {
            console.log(`event has no metadata`);
            return;
        }
        if (cellClicked(metadata)) {
            console.log(`cell clicked`);
        }
        if (columnHeaderClicked(metadata)) {
            console.log(`column header clicked`);
        }
        if (rowClicked(metadata)) {
            console.log(`row clicked`);
        }
        if (rowHeaderClicked(metadata)) {
            console.log(`row header clicked`);
        }
        console.log(metadata);
    }
    RegularTable.clickLoggingListener = clickLoggingListener;
    /*
     * general metadata-related functions
     */
    function colNameFromMeta(meta) {
        const { column_header, value } = meta;
        return (value instanceof HTMLElement ? value.textContent : column_header === "0" ? "path" : value);
    }
    RegularTable.colNameFromMeta = colNameFromMeta;
    function metadataFromElement(target, rt, recursive = true) {
        if (target.tagName === TOP_LEVEL_TAGNAME) {
            return;
        }
        let metadata = rt.getMeta(target);
        if (!recursive || metadata || !target.parentElement) {
            return metadata;
        }
        return metadataFromElement(target.parentElement, rt, recursive);
    }
    RegularTable.metadataFromElement = metadataFromElement;
})(RegularTable || (RegularTable = {}));
export var Path;
(function (Path) {
    function fromarray(x) {
        const [first, ...rest] = x;
        const drive = first ? first + ":" : "";
        const local = rest.join("/");
        return `${drive}${local}`;
    }
    Path.fromarray = fromarray;
    function toarray(x) {
        const splits = x.split(/:/g);
        const [drive, local] = splits.length >= 2 ? splits : ["", x];
        return [drive, ...(local ? local.split(/[\/\\]/g) : [])];
    }
    Path.toarray = toarray;
    /**
     * remove all back/slashes, then add a trailing slash if requested to a string
     */
    function normSlash(x, trailing) {
        return Path.trimSlash(x) + (trailing ? "/" : "");
    }
    Path.normSlash = normSlash;
    /**
     * remove all back/slashes from string
     */
    function trimSlash(x) {
        return x.replace(/[\/\\]/g, "");
    }
    Path.trimSlash = trimSlash;
})(Path || (Path = {}));
export var Tag;
(function (Tag) {
    Tag.html = (strings, ...args) => strings
        .map((str, i) => [str, args[i]])
        .flat()
        .filter((a) => !!a)
        .join("");
})(Tag || (Tag = {}));
export var Tree;
(function (Tree) {
    const treeTemplate = document.createElement("template");
    const inputSpanTemplate = document.createElement("template");
    const nameSpanTemplate = document.createElement("template");
    function breadcrumbsSpans(path) {
        return `<div class="tf-breadcrumbs-home"><span class="tf-breadcrumbs-crumb tf-breadcrumbs-icon tf-breadcrumbs-dir-icon" data-crumbix="0"></span><span>/</span></div>` + ([...path.slice(1),]
            .map((x, i) => `<span class="tf-breadcrumbs-crumb" data-crumbix="${i + 1}">${x}</span>`)
            .join(`<span class="tf-breadcrumbs-separator">/</span>`));
    }
    Tree.breadcrumbsSpans = breadcrumbsSpans;
    function colHeaderSpans(name, filter = false) {
        nameSpanTemplate.innerHTML = `<span class="tf-header"><span class="tf-header-name">${name}</span><span class="tf-header-sort"></span></span>`;
        if (filter) {
            inputSpanTemplate.innerHTML = `<input class="tf-header-input"></input>`;
            return [inputSpanTemplate.content.firstChild, nameSpanTemplate.content.firstChild];
        }
        else {
            return [nameSpanTemplate.content.firstChild];
        }
    }
    Tree.colHeaderSpans = colHeaderSpans;
    function rowHeaderLevelsHtml({ isDir, isOpen, path = [] }) {
        const tree_levels = path.slice(1).map(() => '<span class="rt-tree-group"></span>');
        if (isDir) {
            const group_icon = isOpen ? "remove" : "add";
            const tree_button = `<span class="rt-row-header-icon">${group_icon}</span>`;
            tree_levels.push(tree_button);
        }
        return tree_levels.join("");
    }
    function rowHeaderSpan({ isDir, isOpen, path, pathRender = "tree" }) {
        path = path.map((x, ix) => Path.normSlash(x, ix < (path.length - 1) ? true : isDir));
        const header_classes = !isDir ? "rt-group-name rt-group-leaf" : "rt-group-name";
        const header_text = pathRender === "relative" ? path.join("") : path.slice(-1).join("");
        const tree_levels = rowHeaderLevelsHtml({ isDir, isOpen, path: pathRender === "tree" ? path : [] });
        treeTemplate.innerHTML = `<span class="rt-tree-container">${tree_levels}<span class="${header_classes}">${header_text}</span></span>`;
        if (pathRender === "regular") {
            return [path.slice(0, -1).join(""), treeTemplate.content.firstChild];
        }
        else {
            return [treeTemplate.content.firstChild];
        }
    }
    Tree.rowHeaderSpan = rowHeaderSpan;
})(Tree || (Tree = {}));
//# sourceMappingURL=util.js.map